@@ -3,7 +3,9 @@
 #include "example.h"
 #include <stdio.h>
 
+#ifndef M_PI
 #define M_PI 3.14159265358979323846
+#endif
 
 /* Move the shape to a new location */
 void Shape::move(double dx, double dy) {
@@ -39,12 +41,15 @@ Square* createSquare(double w)
     return new Square(w);
 }
 
-ShapeOwner::ShapeOwner() {printf("  ShapeOwner(%p)\n",this);}
+ShapeOwner::ShapeOwner() {
+  printf("  ShapeOwner(%p)\n", (void *)this);
+}
+
 ShapeOwner::~ShapeOwner()
 {
-    printf("  ~ShapeOwner(%p)\n",this);
-    for(unsigned i=0;i<shapes.size();i++)
-        delete shapes[i];
+  printf("  ~ShapeOwner(%p)\n", (void *)this);
+  for(unsigned i=0;i<shapes.size();i++)
+    delete shapes[i];
 }
 
 void ShapeOwner::add(Shape* ptr) // this method takes ownership of the object
@@ -54,14 +59,14 @@ void ShapeOwner::add(Shape* ptr) // this method takes ownership of the object
 
 Shape* ShapeOwner::get(int idx) // this pointer is still owned by the class (assessor)
 {
-    if (idx<0 || idx>=shapes.size())
+    if (idx < 0 || idx >= static_cast<int>(shapes.size()))
         return NULL;
     return shapes[idx];
 }
 
 Shape* ShapeOwner::remove(int idx) // this method returns memory which must be deleted
 {
-    if (idx<0 || idx>=shapes.size())
+    if (idx < 0 || idx >= static_cast<int>(shapes.size()))
         return NULL;
     Shape* ptr=shapes[idx];
     shapes.erase(shapes.begin()+idx);
