@@ -6,38 +6,38 @@ endif
 li_cwstring
 
 if (count("ab\0ab\0ab\0", 0) != 3)
-    error    
+    error("failed");
 endif
 
 if (!strcmp(test1(),"Hello World"))
-    error
+    error("failed");
 endif
 
 if (!strcmp(test2()," !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_"))
-    error
+    error("failed");
 endif
 
 if (!strcmp(test3("hello"),"hello-suffix"))
-    error
+    error("failed");
 endif
 
 if (!strcmp(test4("hello"),"hello-suffix"))
-    error
+    error("failed");
 endif
     
 if (!strcmp(test5(4),'xxxx'))
-    error
+    error("failed");
 endif
 
 if (!strcmp(test6(10),'xxxxx'))
-    error
+    error("failed");
 endif
     
 if (!strcmp(test7(),"Hello world!"))
-    error
+    error("failed");
 endif
 
 if (!strcmp(test8()," !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_"))
-    error
+    error("failed");
 endif
 
